#include "emu.h"

#include "cpufeature.h"
#include "msr-index.h"
#include "apicdef.h"

/* --------------------------------------------------------------------- */

static uint32_t apic_read(struct xen_cpu *cpu, int reg)
{
    uint32_t *ptr = (cpu->lapic + reg);
    return *ptr;
}

void lapic_init(struct xen_cpu *cpu)
{
    struct kvm_cpuid_entry entry;
    uint32_t ax,dx;
    ureg_t base;

    entry.function = 0x00000001;
    real_cpuid(&entry);
    if (!(entry.edx & (1 << (X86_FEATURE_APIC    %32)))) {
	printk(1, "%s: no lapic present\n", __FUNCTION__);
	return;
    }

    rdmsr(MSR_IA32_APICBASE, &ax, &dx);
    base = (uint64_t)dx << 32 | (ax & PAGE_MASK);
    cpu->lapic = fixmap_page(base);
    printk(1, "%s: base %" PRIxREG "%s%s, mapped to %p\n",
	   __FUNCTION__, base,
	   ax & 0x00000800 ? ", enabled" : "",
	   ax & 0x00000800 ? ", bootcpu" : "",
	   cpu->lapic);
    printk(1, "%s: ver %d, maxlvt %d\n", __FUNCTION__,
	   GET_APIC_VERSION(apic_read(cpu, APIC_LVR)),
	   GET_APIC_MAXLVT(apic_read(cpu, APIC_LVR)));
}
